@@ -16,15 +16,20 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "ephy-types.h"
 #include "ephy-history.h"
 #include "ephy-file-helpers.h"
 #include "ephy-autocompletion-source.h"
 #include "ephy-debug.h"
+#include "ephy-node-common.h"
 
 #include <time.h>
 #include <string.h>
-#include <libgnome/gnome-i18n.h>
+#include <bonobo/bonobo-i18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 
 #define EPHY_HISTORY_XML_VERSION "0.1"
@@ -201,7 +206,7 @@ ephy_history_load (EphyHistory *eb)
 		return;
 
 	doc = xmlParseFile (eb->priv->xml_file);
-	g_assert (doc != NULL);
+	g_return_if_fail (doc != NULL);
 
 	root = xmlDocGetRootElement (doc);
 
@@ -257,7 +262,7 @@ remove_obsolete_pages (EphyHistory *eb)
 
 		if (page_is_obsolete (kid, &current_date))
 		{
-			ephy_history_remove (eb, kid);
+			ephy_node_unref (kid);
 		}
 	}
 }
@@ -286,6 +291,7 @@ ephy_history_save (EphyHistory *eb)
 		EphyNode *kid;
 
 		kid = g_ptr_array_index (children, i);
+		if (kid == eb->priv->pages) continue;
 
 		ephy_node_save_to_xml (kid, root);
 	}
@@ -352,12 +358,27 @@ pages_removed_cb (EphyNode *node,
 		  EphyNode *child,
 		  EphyHistory *eb)
 {
+	EphyNode *host;
+	int host_id;
+	int children;
+
 	g_static_rw_lock_writer_lock (eb->priv->pages_hash_lock);
 
 	g_hash_table_remove (eb->priv->pages_hash,
 			     ephy_node_get_property_string (child, EPHY_NODE_PAGE_PROP_LOCATION));
 
 	g_static_rw_lock_writer_unlock (eb->priv->pages_hash_lock);
+
+	host_id = ephy_node_get_property_int (child, EPHY_NODE_PAGE_PROP_HOST_ID);
+	host = ephy_node_get_from_id (host_id);
+	children = ephy_node_get_n_children (host);
+
+	LOG ("Check host children: %d", children)
+
+	if (children == 0)
+	{
+		ephy_node_unref (host);
+	}
 }
 
 static gboolean
@@ -372,6 +393,8 @@ periodic_save_cb (EphyHistory *eh)
 static void
 ephy_history_init (EphyHistory *eb)
 {
+	GValue value = { 0, };
+
         eb->priv = g_new0 (EphyHistoryPrivate, 1);
 
 	eb->priv->xml_file = g_build_filename (ephy_dot_dir (),
@@ -391,6 +414,21 @@ ephy_history_init (EphyHistory *eb)
 	/* Pages */
 	eb->priv->pages = ephy_node_new_with_id (PAGES_NODE_ID);
 	ephy_node_ref (eb->priv->pages);
+	g_value_init (&value, G_TYPE_STRING);
+	g_value_set_string (&value, _("All"));
+	ephy_node_set_property (eb->priv->pages,
+			        EPHY_NODE_PAGE_PROP_LOCATION,
+			        &value);
+	ephy_node_set_property (eb->priv->pages,
+			        EPHY_NODE_PAGE_PROP_TITLE,
+			        &value);
+	g_value_unset (&value);
+	g_value_init (&value, G_TYPE_INT);
+	g_value_set_int (&value, EPHY_NODE_ALL_PRIORITY);
+	ephy_node_set_property (eb->priv->pages,
+			        EPHY_NODE_PAGE_PROP_PRIORITY,
+			        &value);
+	g_value_unset (&value);
 	g_signal_connect_object (G_OBJECT (eb->priv->pages),
 				 "child_added",
 				 G_CALLBACK (pages_added_cb),
@@ -416,6 +454,8 @@ ephy_history_init (EphyHistory *eb)
 				 G_OBJECT (eb),
 				 0);
 
+	ephy_node_add_child (eb->priv->hosts, eb->priv->pages);
+
 	ephy_history_load (eb);
 	ephy_history_emit_data_changed (eb);
 
@@ -466,33 +506,6 @@ ephy_history_new ()
 	return tab;
 }
 
-static void
-ephy_history_host_set_title (EphyHistory *eh,
-			     EphyNode *host,
-			     EphyNode *page,
-			     const char *title)
-{
-	const char *real_url;
-	const char *host_url;
-	GValue value = { 0, };
-
-	real_url = ephy_node_get_property_string
-		(page, EPHY_NODE_PAGE_PROP_LOCATION);
-	host_url = ephy_node_get_property_string
-		(host, EPHY_NODE_PAGE_PROP_LOCATION);
-
-	if (real_url && host_url &&
-	    strcmp (real_url, host_url) == 0)
-	{
-
-		g_value_init (&value, G_TYPE_STRING);
-		g_value_set_string (&value, title);
-		ephy_node_set_property (host, EPHY_NODE_PAGE_PROP_TITLE,
-					&value);
-		g_value_unset (&value);
-	}
-}
-
 static void
 ephy_history_host_visited (EphyHistory *eh,
 			   EphyNode *host,
@@ -764,11 +777,12 @@ ephy_history_set_page_title (EphyHistory *gh,
 			     const char *title)
 {
 	EphyNode *node;
-	guint host_id;
 	GValue value = { 0, };
 
 	LOG ("Set page title")
 
+	if (title == NULL || title[0] == '\0') return;
+
 	node = ephy_history_get_page (gh, url);
 	if (!node) return;
 
@@ -777,13 +791,27 @@ ephy_history_set_page_title (EphyHistory *gh,
 	ephy_node_set_property
 		(node, EPHY_NODE_PAGE_PROP_TITLE, &value);
 	g_value_unset (&value);
+}
 
-	host_id = ephy_node_get_property_int
-		(node, EPHY_NODE_PAGE_PROP_HOST_ID);
-	if (host_id >= 0)
+void
+ephy_history_set_icon (EphyHistory *gh,
+		       const char *url,
+		       const char *icon)
+{
+	EphyNode *host;
+
+	LOG ("Set host icon")
+
+	host = g_hash_table_lookup (gh->priv->hosts_hash, url);
+	if (host)
 	{
-		ephy_history_host_set_title (gh, ephy_node_get_from_id  (host_id),
-					     node, title);
+		GValue value = { 0, };
+
+		g_value_init (&value, G_TYPE_STRING);
+		g_value_set_string (&value, icon);
+		ephy_node_set_property
+			(host, EPHY_NODE_PAGE_PROP_ICON, &value);
+		g_value_unset (&value);
 	}
 }
 
@@ -797,7 +825,8 @@ ephy_history_clear (EphyHistory *gh)
 		ephy_node_unref (node);
 	}
 
-	while ((node = ephy_node_get_nth_child (gh->priv->hosts, 0)) != NULL)
+	/* The first node is All, dont unref it */
+	while ((node = ephy_node_get_nth_child (gh->priv->hosts, 1)) != NULL)
 	{
 		ephy_node_unref (node);
 	}
@@ -825,34 +854,3 @@ ephy_history_get_last_page (EphyHistory *gh)
 	return ephy_node_get_property_string
 		(gh->priv->last_page, EPHY_NODE_PAGE_PROP_LOCATION);
 }
-
-void
-ephy_history_remove (EphyHistory *gh, EphyNode *node)
-{
-	EphyNode *host;
-	int host_id;
-
-	LOG ("Remove history item")
-
-	host_id = ephy_node_get_property_int (node, EPHY_NODE_PAGE_PROP_HOST_ID);
-	if (host_id < 0)
-	{
-		EphyNode *tmp;
-		while ((tmp = ephy_node_get_nth_child (node, 0)) != NULL)
-		{
-			ephy_node_unref (tmp);
-		}
-		ephy_node_unref (node);
-		return;
-	}
-
-	host = ephy_node_get_from_id (host_id);
-	if (ephy_node_get_n_children (host) == 1)
-	{
-		ephy_node_unref (host);
-	}
-	else
-	{
-		ephy_node_unref (node);
-	}
-}
