@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-file-helpers.c,v 1.2 2003/01/07 20:34:09 mpeseng Exp $
+ *  $Id: ephy-file-helpers.c,v 1.3 2003/05/17 12:43:27 cneumair Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,7 +27,7 @@
 #include <unistd.h>
 #include <sys/stat.h>
 #include <glib.h>
-#include <libgnome/gnome-i18n.h>
+#include <bonobo/bonobo-i18n.h>
 #include <libgnome/gnome-init.h>
 #include <libgnome/gnome-exec.h>
 
