@@ -17,12 +17,19 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "ephy-bookmark-properties.h"
 #include "ephy-topics-selector.h"
 #include "ephy-debug.h"
 #include "ephy-shell.h"
 #include "ephy-state.h"
+#include "ephy-gui.h"
 
+#include <gtk/gtkcheckbutton.h>
+#include <gtk/gtktogglebutton.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkvbox.h>
@@ -30,7 +37,7 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmisc.h>
 #include <gtk/gtkscrolledwindow.h>
-#include <libgnome/gnome-i18n.h>
+#include <bonobo/bonobo-i18n.h>
 
 static void ephy_bookmark_properties_class_init (EphyBookmarkPropertiesClass *klass);
 static void ephy_bookmark_properties_init (EphyBookmarkProperties *editor);
@@ -52,6 +59,8 @@ struct EphyBookmarkPropertiesPrivate
 	GtkWidget *title_entry;
 	GtkWidget *location_entry;
 	GtkWidget *topics_selector;
+
+	EphyToolbarsModel *tb_model;
 };
 
 enum
@@ -197,6 +206,11 @@ bookmark_properties_response_cb (GtkDialog *dialog,
 {
 	switch (response_id)
 	{
+		case GTK_RESPONSE_HELP:
+			ephy_gui_help (GTK_WINDOW (dialog),
+				       "epiphany", 
+				       "to-edit-bookmark-properties");
+			break;
 		case GTK_RESPONSE_CLOSE:
 			gtk_widget_destroy (GTK_WIDGET (dialog));
 			break;
@@ -224,7 +238,7 @@ update_window_title(EphyBookmarkProperties *editor)
 {
 	char *title;
 	const char *tmp;
-	
+
 	tmp = ephy_node_get_property_string (editor->priv->bookmark,
 					     EPHY_NODE_BMK_PROP_TITLE);
 	title = g_strdup_printf (_("%s Properties"), tmp);
@@ -246,6 +260,28 @@ location_entry_changed_cb (GtkWidget *entry, EphyBookmarkProperties *props)
 	update_entry (props, entry, EPHY_NODE_BMK_PROP_LOCATION);
 }
 
+static void
+toolbar_checkbox_changed_cb (GtkWidget *checkbox, EphyBookmarkProperties *props)
+{
+	gboolean state;
+	gulong id;
+
+	state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox));
+
+	id = ephy_node_get_id (props->priv->bookmark);
+
+	if (state)
+	{
+		ephy_toolbars_model_add_bookmark
+			(props->priv->tb_model, FALSE, id);
+	}
+	else
+	{
+		ephy_toolbars_model_remove_bookmark
+			(props->priv->tb_model, FALSE, id);
+	}
+}
+
 static void
 set_window_icon (EphyBookmarkProperties *editor)
 {
@@ -266,10 +302,10 @@ set_window_icon (EphyBookmarkProperties *editor)
 
 	else
 	{
-		icon = gtk_widget_render_icon (GTK_WIDGET (editor), 
-						      	      GTK_STOCK_PROPERTIES,
-						      	      GTK_ICON_SIZE_MENU,
-						      	      NULL);
+		icon = gtk_widget_render_icon (GTK_WIDGET (editor),
+					       GTK_STOCK_PROPERTIES,
+					       GTK_ICON_SIZE_MENU,
+					       NULL);
 	}
 
 	gtk_window_set_icon (GTK_WINDOW (editor), icon);
@@ -279,9 +315,12 @@ set_window_icon (EphyBookmarkProperties *editor)
 static void
 build_ui (EphyBookmarkProperties *editor)
 {
-	GtkWidget *table, *label, *entry, *topics_selector, *scrolled_window;
+	GtkWidget *table, *label, *entry, *topics_selector;
+	GtkWidget *checkbox, *scrolled_window;
 	char *str;
 	const char *tmp;
+	gboolean state;
+	gulong id;
 
 	g_signal_connect (G_OBJECT (editor),
 			  "response",
@@ -290,7 +329,8 @@ build_ui (EphyBookmarkProperties *editor)
 
 	ephy_state_add_window (GTK_WIDGET(editor),
 			       "bookmark_properties",
-		               290, 280);
+		               290, 280,
+			       EPHY_STATE_WINDOW_SAVE_SIZE);
 
 	update_window_title (editor);
 	set_window_icon (editor);
@@ -303,7 +343,7 @@ build_ui (EphyBookmarkProperties *editor)
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
 	gtk_widget_show (table);
-	
+
 	entry = gtk_entry_new ();
 	tmp = ephy_node_get_property_string (editor->priv->bookmark,
 					     EPHY_NODE_BMK_PROP_TITLE);
@@ -335,7 +375,7 @@ build_ui (EphyBookmarkProperties *editor)
 	gtk_widget_show (entry);
 	label = gtk_label_new (NULL);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	str = g_strconcat ("<b>", _("_Location:"), "</b>", NULL);
+	str = g_strconcat ("<b>", _("_Address:"), "</b>", NULL);
 	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str);
 	g_free (str);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
@@ -367,9 +407,21 @@ build_ui (EphyBookmarkProperties *editor)
 	gtk_table_attach (GTK_TABLE (table), scrolled_window, 1, 2, 2, 3,
 			  GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 
+	checkbox = gtk_check_button_new_with_mnemonic (_("_Show in bookmarks bar"));
+	id = ephy_node_get_id (editor->priv->bookmark);
+	state = ephy_toolbars_model_has_bookmark (editor->priv->tb_model, FALSE, id);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox), state);
+	g_signal_connect (checkbox, "toggled",
+			  G_CALLBACK (toolbar_checkbox_changed_cb), editor);
+	gtk_table_attach (GTK_TABLE (table), checkbox, 0, 2, 3, 4, GTK_FILL, 0, 0, 0);
+	gtk_widget_show (checkbox);
+
+
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (editor)->vbox),
 			    table, TRUE, TRUE, 0);
-
+	gtk_dialog_add_button (GTK_DIALOG (editor),
+			       GTK_STOCK_HELP,
+			       GTK_RESPONSE_HELP);
 	gtk_dialog_add_button (GTK_DIALOG (editor),
 			       GTK_STOCK_CLOSE,
 			       GTK_RESPONSE_CLOSE);
@@ -382,6 +434,7 @@ ephy_bookmark_properties_init (EphyBookmarkProperties *editor)
 	editor->priv = g_new0 (EphyBookmarkPropertiesPrivate, 1);
 
 	editor->priv->bookmark = NULL;
+	editor->priv->tb_model = ephy_shell_get_toolbars_model (ephy_shell);
 }
 
 GtkWidget *
@@ -407,3 +460,10 @@ ephy_bookmark_properties_new (EphyBookmarks *bookmarks,
 	}
 	return GTK_WIDGET (editor);
 }
+
+EphyNode *
+ephy_bookmark_properties_get_node (EphyBookmarkProperties *properties)
+{
+	return properties->priv->bookmark;
+}
+
