@@ -51,7 +51,7 @@ struct EphyTabPrivate
 	gboolean is_active;
 	TabLoadStatus load_status;
 	char status_message[255];
-	char link_message[255];
+	char *link_message;
 	char favicon_url[255];
 	char *title;
 	char *location;
@@ -63,12 +63,9 @@ struct EphyTabPrivate
 	int height;
 };
 
-static void
-ephy_tab_class_init (EphyTabClass *klass);
-static void
-ephy_tab_init (EphyTab *tab);
-static void
-ephy_tab_finalize (GObject *object);
+static void ephy_tab_class_init (EphyTabClass *klass);
+static void ephy_tab_init (EphyTab *tab);
+static void ephy_tab_finalize (GObject *object);
 
 enum
 {
@@ -76,57 +73,6 @@ enum
 	PROP_EPHY_SHELL
 };
 
-static void
-ephy_tab_set_favicon (EphyTab *tab,
-		      GdkPixbuf *favicon);
-static void
-ephy_tab_favicon_cb (EphyEmbed *embed,
-		     const char *url,
-		     EphyTab *tab);
-static void
-ephy_tab_favicon_cache_changed_cb (EphyFaviconCache *cache, 
-				   char *url, 
-				   EphyTab *tab);
-static void
-ephy_tab_link_message_cb (EphyEmbed *embed,
-			  const char *message,
-			  EphyTab *tab);
-static void
-ephy_tab_location_cb (EphyEmbed *embed, EphyTab *tab);
-static void
-ephy_tab_title_cb (EphyEmbed *embed, EphyTab *tab);
-static void
-ephy_tab_net_state_cb (EphyEmbed *embed, const char *uri,
-		       EmbedState state, EphyTab *tab);
-static void
-ephy_tab_new_window_cb (EphyEmbed *embed, EphyEmbed **new_embed,
-			EmbedChromeMask chromemask, EphyTab *tab);
-static void
-ephy_tab_visibility_cb (EphyEmbed *embed, gboolean visibility,
-			EphyTab *tab);
-static void
-ephy_tab_destroy_brsr_cb (EphyEmbed *embed, EphyTab *tab);
-static gint
-ephy_tab_open_uri_cb (EphyEmbed *embed, const char *uri,
-		      EphyTab *tab);
-static void
-ephy_tab_size_to_cb (EphyEmbed *embed, gint width, gint height,
-		     EphyTab *tab);
-static gint
-ephy_tab_dom_mouse_click_cb (EphyEmbed *embed,
-			     EphyEmbedEvent *event,
-			     EphyTab *tab);
-static gint
-ephy_tab_dom_mouse_down_cb (EphyEmbed *embed,
-			    EphyEmbedEvent *event,
-			    EphyTab *tab);
-static void
-ephy_tab_security_change_cb (EphyEmbed *embed, EmbedSecurityLevel level,
-			     EphyTab *tab);
-static void
-ephy_tab_zoom_changed_cb (EphyEmbed *embed, gint zoom,
-			  EphyTab *tab);
-
 static GObjectClass *parent_class = NULL;
 
 /* Class functions */
@@ -189,101 +135,6 @@ ephy_tab_embed_destroy_cb (GtkWidget *widget, EphyTab *tab)
 	g_object_unref (tab);
 }
 
-static void
-ephy_tab_init (EphyTab *tab)
-{
-	GObject *embed, *embed_widget;
-	EphyEmbedSingle *single;
-	EphyFaviconCache *cache;
-
-	single = ephy_embed_shell_get_embed_single
-		(EPHY_EMBED_SHELL (ephy_shell));
-
-        tab->priv = g_new0 (EphyTabPrivate, 1);
-
-	tab->priv->embed = ephy_embed_new (G_OBJECT(single));
-	ephy_embed_shell_add_embed (EPHY_EMBED_SHELL (ephy_shell),
-				    tab->priv->embed);
-
-	tab->priv->window = NULL;
-	tab->priv->event = NULL;
-	tab->priv->is_active = FALSE;
-	*tab->priv->status_message = '\0';
-	*tab->priv->link_message = '\0';
-	*tab->priv->favicon_url = '\0';
-	tab->priv->load_status = TAB_LOAD_NONE;
-	tab->priv->load_percent = 0;
-	tab->priv->title = NULL;
-	tab->priv->location = NULL;
-	tab->priv->total_requests = 0;
-	tab->priv->cur_requests = 0;
-	tab->priv->width = -1;
-	tab->priv->height = -1;
-
-
-	embed = G_OBJECT (tab->priv->embed);
-	embed_widget = G_OBJECT (tab->priv->embed);
-
-	/* set a pointer in the embed's widget back to the tab */
-	g_object_set_data (embed_widget, "EphyTab", tab);
-
-	g_signal_connect (embed_widget, "parent_set",
-			  G_CALLBACK (ephy_tab_parent_set_cb),
-			  tab);
-	g_signal_connect (embed_widget, "destroy",
-			  GTK_SIGNAL_FUNC (ephy_tab_embed_destroy_cb),
-			  tab);
-	g_signal_connect (embed, "ge_link_message",
-			  GTK_SIGNAL_FUNC (ephy_tab_link_message_cb),
-			  tab);
-	g_signal_connect (embed, "ge_location",
-			  GTK_SIGNAL_FUNC (ephy_tab_location_cb),
-			  tab);
-	g_signal_connect (embed, "ge_title",
-			  GTK_SIGNAL_FUNC (ephy_tab_title_cb),
-			  tab);
-	g_signal_connect (embed, "ge_zoom_change",
-			  GTK_SIGNAL_FUNC (ephy_tab_zoom_changed_cb),
-			  tab);
-	g_signal_connect (embed, "ge_net_state",
-			  GTK_SIGNAL_FUNC (ephy_tab_net_state_cb),
-			  tab);
-	g_signal_connect (embed, "ge_new_window",
-			  GTK_SIGNAL_FUNC (ephy_tab_new_window_cb),
-			  tab);
-	g_signal_connect (embed, "ge_visibility",
-			  GTK_SIGNAL_FUNC (ephy_tab_visibility_cb),
-			  tab);
-	g_signal_connect (embed, "ge_destroy_brsr",
-			  GTK_SIGNAL_FUNC (ephy_tab_destroy_brsr_cb),
-			  tab);
-	g_signal_connect (embed, "ge_open_uri",
-			  GTK_SIGNAL_FUNC (ephy_tab_open_uri_cb),
-			  tab);
-	g_signal_connect (embed, "ge_size_to",
-			  GTK_SIGNAL_FUNC (ephy_tab_size_to_cb),
-			  tab);
-	g_signal_connect (embed, "ge_dom_mouse_click",
-			  GTK_SIGNAL_FUNC (ephy_tab_dom_mouse_click_cb),
-			  tab);
-	g_signal_connect (embed, "ge_dom_mouse_down",
-			  GTK_SIGNAL_FUNC (ephy_tab_dom_mouse_down_cb),
-			  tab);
-	g_signal_connect (embed, "ge_security_change",
-			  GTK_SIGNAL_FUNC (ephy_tab_security_change_cb),
-			  tab);
-	g_signal_connect (embed, "ge_favicon",
-			  GTK_SIGNAL_FUNC (ephy_tab_favicon_cb),
-			  tab);
-
-	cache = ephy_embed_shell_get_favicon_cache (EPHY_EMBED_SHELL (ephy_shell));
-	g_signal_connect_object (G_OBJECT (cache), "changed",
-				 G_CALLBACK (ephy_tab_favicon_cache_changed_cb),
-				 tab,  0);
-}
-
-/* Destructor */
-
 static void
 ephy_tab_finalize (GObject *object)
 {
@@ -303,6 +154,7 @@ ephy_tab_finalize (GObject *object)
 	}
 
 	g_free (tab->priv->location);
+	g_free (tab->priv->link_message);
 
         g_free (tab->priv);
 
@@ -451,31 +303,37 @@ ephy_tab_set_visibility (EphyTab *tab,
 }
 
 static void
-ephy_tab_set_favicon (EphyTab *tab, 
+ephy_tab_set_favicon (EphyTab *tab,
 		      GdkPixbuf *favicon)
 {
 	GtkWidget *nb;
 	EphyBookmarks *eb;
+	EphyHistory *history;
 
 	nb = ephy_window_get_notebook (tab->priv->window);
 	ephy_notebook_set_page_icon (EPHY_NOTEBOOK (nb),
 				     GTK_WIDGET (tab->priv->embed),
 				     favicon);
 
-	if (!tab->priv->is_active) return;
-
-	eb = ephy_shell_get_bookmarks (ephy_shell);
-	ephy_bookmarks_set_icon (eb, tab->priv->location,
-			         tab->priv->favicon_url);
-	ephy_window_update_control (tab->priv->window,
-				    FaviconControl);
+	if (tab->priv->favicon_url[0] != '\0')
+	{
+		eb = ephy_shell_get_bookmarks (ephy_shell);
+		history = ephy_embed_shell_get_global_history
+			(EPHY_EMBED_SHELL (ephy_shell));
+		ephy_bookmarks_set_icon (eb, tab->priv->location,
+				         tab->priv->favicon_url);
+		ephy_history_set_icon (history, tab->priv->location,
+				       tab->priv->favicon_url);
+		ephy_window_update_control (tab->priv->window,
+					    FaviconControl);
+	}
 }
 
 /* Private callbacks for embed signals */
 
 static void
-ephy_tab_favicon_cache_changed_cb (EphyFaviconCache *cache, 
-				   char *url, 
+ephy_tab_favicon_cache_changed_cb (EphyFaviconCache *cache,
+				   char *url,
 				   EphyTab *tab)
 {
 	GdkPixbuf *pixbuf = NULL;
@@ -486,7 +344,7 @@ ephy_tab_favicon_cache_changed_cb (EphyFaviconCache *cache,
 	/* set favicon */
 	pixbuf = ephy_favicon_cache_get (cache, tab->priv->favicon_url);
 	ephy_tab_set_favicon (tab, pixbuf);
-	
+
 	if (pixbuf) g_object_unref (pixbuf);
 }
 
@@ -523,8 +381,8 @@ ephy_tab_link_message_cb (EphyEmbed *embed,
 {
 	if (!tab->priv->is_active) return;
 
-	g_strlcpy (tab->priv->link_message,
-		   message, 255);
+	g_free (tab->priv->link_message);
+	tab->priv->link_message = g_strdup (message);
 
 	ephy_window_update_control (tab->priv->window,
 				    StatusbarMessageControl);
@@ -536,8 +394,7 @@ ephy_tab_location_cb (EphyEmbed *embed, EphyTab *tab)
 	if (tab->priv->location) g_free (tab->priv->location);
 	ephy_embed_get_location (embed, TRUE,
 				 &tab->priv->location);
-	tab->priv->link_message[0] = '\0';
-
+	tab->priv->link_message = NULL;
 	tab->priv->favicon_url[0] = '\0';
 	ephy_tab_set_favicon (tab, NULL);
 
@@ -572,13 +429,33 @@ ephy_tab_set_title (EphyTab *tab, const char *title)
 static void
 ephy_tab_title_cb (EphyEmbed *embed, EphyTab *tab)
 {
+	GnomeVFSURI *uri;
+
 	if (tab->priv->title) g_free (tab->priv->title);
 	ephy_embed_get_title (embed, &tab->priv->title);
 
 	if (*(tab->priv->title) == '\0')
 	{
 		g_free (tab->priv->title);
-		tab->priv->title = g_strdup (_("Untitled"));
+		tab->priv->title = NULL;
+
+		uri = gnome_vfs_uri_new (tab->priv->location);
+		if (uri)
+		{
+			tab->priv->title = gnome_vfs_uri_to_string (uri,
+						GNOME_VFS_URI_HIDE_USER_NAME |
+						GNOME_VFS_URI_HIDE_PASSWORD |
+						GNOME_VFS_URI_HIDE_HOST_PORT |
+						GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD |
+						GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER);
+			gnome_vfs_uri_unref (uri);
+		}
+
+		if (tab->priv->title == NULL || *(tab->priv->title) == '\0')
+		{
+			g_free (tab->priv->title);
+			tab->priv->title = g_strdup (_("Blank page"));
+		}
 	}
 
 	ephy_tab_set_title (tab, tab->priv->title);
@@ -769,24 +646,12 @@ ephy_tab_new_window_cb (EphyEmbed *embed, EphyEmbed **new_embed,
 {
 	EphyTab *new_tab;
 	EphyWindow *window;
-	gboolean open_in_tab;
 
-	open_in_tab = (chromemask & EMBED_CHROME_OPENASCHROME) ?
-		      FALSE :
-		      eel_gconf_get_boolean (CONF_TABS_TABBED_POPUPS);
-
-	if (open_in_tab)
-	{
-		window = ephy_tab_get_window (tab);
-	}
-	else
-	{
-		window = ephy_window_new ();
-		ephy_window_set_chrome (window, chromemask);
-	}
+	window = ephy_window_new ();
+	ephy_window_set_chrome (window, chromemask);
 
 	new_tab = ephy_tab_new ();
-        ephy_window_add_tab (window, new_tab, FALSE);
+        ephy_window_add_tab (window, new_tab, EPHY_NOTEBOOK_INSERT_GROUPED, FALSE);
 
 	*new_embed = ephy_tab_get_embed (new_tab);
 }
@@ -882,14 +747,6 @@ ephy_tab_size_to_cb (EphyEmbed *embed, gint width, gint height,
 	}
 }
 
-static gint
-ephy_tab_dom_mouse_click_cb (EphyEmbed *embed,
-			       EphyEmbedEvent *event,
-			       EphyTab *tab)
-{
-	return FALSE;
-}
-
 static void
 ephy_tab_set_event (EphyTab *tab,
 		    EphyEmbedEvent *event)
@@ -953,9 +810,9 @@ ephy_tab_show_embed_popup (EphyTab *tab, EphyEmbedEvent *event)
 }
 
 static gint
-ephy_tab_dom_mouse_down_cb  (EphyEmbed *embed,
-			     EphyEmbedEvent *event,
-			     EphyTab *tab)
+ephy_tab_dom_mouse_click_cb  (EphyEmbed *embed,
+			      EphyEmbedEvent *event,
+			      EphyTab *tab)
 {
 	EphyWindow *window;
 	int button;
@@ -969,12 +826,8 @@ ephy_tab_dom_mouse_down_cb  (EphyEmbed *embed,
 	ephy_embed_event_get_mouse_button (event, &button);
 	ephy_embed_event_get_context (event, &context);
 
-	if (button == 2)
-	{
-		ephy_tab_show_embed_popup (tab, event);
-	}
-	else if (button == 1
-		 && (context & EMBED_CONTEXT_LINK))
+	if (button == 1
+	    && (context & EMBED_CONTEXT_LINK))
 	{
 		const GValue *value;
 
@@ -982,10 +835,11 @@ ephy_tab_dom_mouse_down_cb  (EphyEmbed *embed,
 		ephy_shell_new_tab (ephy_shell, window, tab,
 				      g_value_get_string (value), 0);
 	}
-	else if (button == 1
-		&& !(context & EMBED_CONTEXT_LINK
-		     || context & EMBED_CONTEXT_EMAIL_LINK
-		     || context & EMBED_CONTEXT_INPUT))
+	else if (button == 1 &&
+		 eel_gconf_get_boolean (CONF_INTERFACE_MIDDLE_CLICK_OPEN_URL) &&
+		 !(context & EMBED_CONTEXT_LINK
+		   || context & EMBED_CONTEXT_EMAIL_LINK
+		   || context & EMBED_CONTEXT_INPUT))
 	{
 		/* paste url */
 		gtk_selection_convert (GTK_WIDGET (window),
@@ -997,6 +851,29 @@ ephy_tab_dom_mouse_down_cb  (EphyEmbed *embed,
 	return FALSE;
 }
 
+static gint
+ephy_tab_dom_mouse_down_cb  (EphyEmbed *embed,
+			     EphyEmbedEvent *event,
+			     EphyTab *tab)
+{
+	EphyWindow *window;
+	int button;
+
+	g_assert (IS_EPHY_EMBED_EVENT(event));
+
+	window = ephy_tab_get_window (tab);
+	g_return_val_if_fail (window != NULL, FALSE);
+
+	ephy_embed_event_get_mouse_button (event, &button);
+
+	if (button == 2)
+	{
+		ephy_tab_show_embed_popup (tab, event);
+	}
+
+	return FALSE;
+}
+
 static void
 ephy_tab_security_change_cb (EphyEmbed *embed, EmbedSecurityLevel level,
 			       EphyTab *tab)
@@ -1007,6 +884,98 @@ ephy_tab_security_change_cb (EphyEmbed *embed, EmbedSecurityLevel level,
 				      StatusbarSecurityControl);
 }
 
+static void
+ephy_tab_init (EphyTab *tab)
+{
+	GObject *embed, *embed_widget;
+	EphyEmbedSingle *single;
+	EphyFaviconCache *cache;
+
+	single = ephy_embed_shell_get_embed_single
+		(EPHY_EMBED_SHELL (ephy_shell));
+
+        tab->priv = g_new0 (EphyTabPrivate, 1);
+
+	tab->priv->window = NULL;
+	tab->priv->event = NULL;
+	tab->priv->is_active = FALSE;
+	*tab->priv->status_message = '\0';
+	tab->priv->link_message = NULL;
+	*tab->priv->favicon_url = '\0';
+	tab->priv->load_status = TAB_LOAD_NONE;
+	tab->priv->load_percent = 0;
+	tab->priv->title = NULL;
+	tab->priv->location = NULL;
+	tab->priv->total_requests = 0;
+	tab->priv->cur_requests = 0;
+	tab->priv->width = -1;
+	tab->priv->height = -1;
+
+	tab->priv->embed = ephy_embed_new (G_OBJECT(single));
+	ephy_embed_shell_add_embed (EPHY_EMBED_SHELL (ephy_shell),
+				    tab->priv->embed);
+
+	embed = G_OBJECT (tab->priv->embed);
+	embed_widget = G_OBJECT (tab->priv->embed);
+
+	/* set a pointer in the embed's widget back to the tab */
+	g_object_set_data (embed_widget, "EphyTab", tab);
+
+	g_signal_connect (embed_widget, "parent_set",
+			  G_CALLBACK (ephy_tab_parent_set_cb),
+			  tab);
+	g_signal_connect (embed_widget, "destroy",
+			  G_CALLBACK (ephy_tab_embed_destroy_cb),
+			  tab);
+	g_signal_connect (embed, "ge_link_message",
+			  G_CALLBACK (ephy_tab_link_message_cb),
+			  tab);
+	g_signal_connect (embed, "ge_location",
+			  G_CALLBACK (ephy_tab_location_cb),
+			  tab);
+	g_signal_connect (embed, "ge_title",
+			  G_CALLBACK (ephy_tab_title_cb),
+			  tab);
+	g_signal_connect (embed, "ge_zoom_change",
+			  G_CALLBACK (ephy_tab_zoom_changed_cb),
+			  tab);
+	g_signal_connect (embed, "ge_net_state",
+			  G_CALLBACK (ephy_tab_net_state_cb),
+			  tab);
+	g_signal_connect (embed, "ge_new_window",
+			  G_CALLBACK (ephy_tab_new_window_cb),
+			  tab);
+	g_signal_connect (embed, "ge_visibility",
+			  G_CALLBACK (ephy_tab_visibility_cb),
+			  tab);
+	g_signal_connect (embed, "ge_destroy_brsr",
+			  G_CALLBACK (ephy_tab_destroy_brsr_cb),
+			  tab);
+	g_signal_connect (embed, "ge_open_uri",
+			  G_CALLBACK (ephy_tab_open_uri_cb),
+			  tab);
+	g_signal_connect (embed, "ge_size_to",
+			  G_CALLBACK (ephy_tab_size_to_cb),
+			  tab);
+	g_signal_connect (embed, "ge_dom_mouse_down",
+			  G_CALLBACK (ephy_tab_dom_mouse_down_cb),
+			  tab);
+	g_signal_connect (embed, "ge_dom_mouse_click",
+			  G_CALLBACK (ephy_tab_dom_mouse_click_cb),
+			  tab);
+	g_signal_connect (embed, "ge_security_change",
+			  G_CALLBACK (ephy_tab_security_change_cb),
+			  tab);
+	g_signal_connect (embed, "ge_favicon",
+			  G_CALLBACK (ephy_tab_favicon_cb),
+			  tab);
+
+	cache = ephy_embed_shell_get_favicon_cache (EPHY_EMBED_SHELL (ephy_shell));
+	g_signal_connect_object (G_OBJECT (cache), "changed",
+				 G_CALLBACK (ephy_tab_favicon_cache_changed_cb),
+				 tab,  0);
+}
+
 TabLoadStatus
 ephy_tab_get_load_status (EphyTab *tab)
 {
@@ -1022,7 +991,7 @@ ephy_tab_get_load_percent (EphyTab *tab)
 const char *
 ephy_tab_get_status_message (EphyTab *tab)
 {
-	if (*tab->priv->link_message)
+	if (tab->priv->link_message)
 	{
 		return tab->priv->link_message;
 	}
