@@ -17,6 +17,10 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "toolbar.h"
 #include "egg-menu-merge.h"
 #include "ephy-file-helpers.h"
@@ -35,8 +39,8 @@
 #include "ephy-string.h"
 #include "ephy-debug.h"
 #include "ephy-new-bookmark.h"
-#include "egg-toolbars-group.h"
 #include "ephy-stock-icons.h"
+#include "eggtoolbar.h"
 
 #include <string.h>
 
@@ -55,6 +59,12 @@ toolbar_get_property (GObject *object,
                       GValue *value,
                       GParamSpec *pspec);
 
+static GtkTargetEntry drag_targets[] = {
+  { EGG_TOOLBAR_ITEM_TYPE, 0, 0 },
+  { EPHY_DND_TOPIC_TYPE, 0, 1 },
+  { EPHY_DND_BOOKMARK_TYPE, 0, 2 },
+};
+static int n_drag_targets = G_N_ELEMENTS (drag_targets);
 
 enum
 {
@@ -62,8 +72,6 @@ enum
 	PROP_EPHY_WINDOW
 };
 
-static EggToolbarsGroup *toolbars_group = NULL;
-
 static GObjectClass *parent_class = NULL;
 
 struct ToolbarPrivate
@@ -118,8 +126,9 @@ go_location_cb (EggAction *action, char *location, EphyWindow *window)
 	ephy_embed_load_url (embed, location);
 }
 
+
 static EggAction *
-get_bookmark_action (Toolbar *t, EphyBookmarks *bookmarks, gulong id, const char *action_name)
+ensure_bookmark_action (Toolbar *t, EphyBookmarks *bookmarks, gulong id, const char *action_name)
 {
 	EggAction *action;
 
@@ -135,110 +144,38 @@ get_bookmark_action (Toolbar *t, EphyBookmarks *bookmarks, gulong id, const char
 	return action;
 }
 
-static char *
-toolbar_get_action_name (EggEditableToolbar *etoolbar,
-		         const char *drag_type,
-			 const char *data)
+static void
+toolbar_ensure_action (Toolbar *t,
+		       const char *name)
 {
-	Toolbar *t = TOOLBAR (etoolbar);
-	EphyBookmarks *bookmarks;
-	gulong id = 0;
-	char *res = NULL;
-
-	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
-
-	if (drag_type && (strcmp (drag_type, EPHY_DND_TOPIC_TYPE) == 0))
-	{
-		GList *nodes;
-		int id;
-
-		nodes = ephy_dnd_node_list_extract_nodes (data);
-		id = ephy_node_get_id (EPHY_NODE (nodes->data));
-		res = g_strdup_printf ("GoTopicId%d", id);
-		g_list_free (nodes);
-	}
-	else if (drag_type && (strcmp (drag_type, EPHY_DND_URL_TYPE) == 0))
-	{
-		GtkWidget *new_bookmark;
-		const char *url;
-		const char *title = NULL;
-		GList *uris;
-
-		uris = ephy_dnd_uri_list_extract_uris ((char *)data);
-		g_return_val_if_fail (uris != NULL, NULL);
-		url = (const char *)uris->data;
-		if (uris->next)
-		{
-			title = (const char *)uris->next->data;
-		}
-
-		LOG ("Action for bookmark -%s-. EphyBookmarks %p", url, bookmarks)
-		id = ephy_bookmarks_get_bookmark_id (bookmarks, url);
-
-		if (id == 0)
-		{
-			new_bookmark = ephy_new_bookmark_new
-				(bookmarks, GTK_WINDOW (t->priv->window), url);
-			ephy_new_bookmark_set_title (EPHY_NEW_BOOKMARK (new_bookmark),
-						     title);
-			gtk_dialog_run (GTK_DIALOG (new_bookmark));
-			id = ephy_new_bookmark_get_id (EPHY_NEW_BOOKMARK (new_bookmark));
-			gtk_widget_destroy (new_bookmark);
-		}
-
-		g_list_foreach (uris, (GFunc)g_free, NULL);
-		g_list_free (uris);
-
-		if (id != 0)
-		{
-			res = g_strdup_printf ("GoBookmarkId%ld", id);
-		}
-		else
-		{
-			res = NULL;
-		}
-	}
-
-	LOG ("Action name is %s", res)
-
-	return res;
-}
-
-static EggAction *
-toolbar_get_action (EggEditableToolbar *etoolbar,
-		    const char *name)
-{
-	Toolbar *t = TOOLBAR (etoolbar);
 	EggAction *action = NULL;
 	gulong id = 0;
 	EphyBookmarks *bookmarks;
 
+	LOG ("Ensure action %s", name)
+
 	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
 
-	action = EGG_EDITABLE_TOOLBAR_CLASS
-			(parent_class)->get_action (etoolbar, name);
-	if (action)
+	if (g_str_has_prefix (name, "GoBookmarkId"))
 	{
-		return action;
-	}
-	else if (g_str_has_prefix (name, "GoBookmarkId"))
-	{
-		LOG ("Create action %s", name)
 		if (!ephy_str_to_int (name + strlen ("GoBookmarkId"), &id))
 		{
-			return NULL;
+			return;
 		}
 
-		action = get_bookmark_action (t, bookmarks, id, name);
+		LOG ("Create action %s", name)
+
+		action = ensure_bookmark_action (t, bookmarks, id, name);
 	}
 	else if (g_str_has_prefix (name, "GoTopicId"))
 	{
-		LOG ("Create action %s", name)
 		if (!ephy_str_to_int (name + strlen ("GoTopicId"), &id))
 		{
-			return NULL;
+			return;
 		}
 
+		LOG ("Create action %s", name)
+
 		action = ephy_topic_action_new (name, id);
 		g_signal_connect (action, "go_location",
 				  G_CALLBACK (go_location_cb),
@@ -246,8 +183,6 @@ toolbar_get_action (EggEditableToolbar *etoolbar,
 		egg_action_group_add_action (t->priv->action_group, action);
 		g_object_unref (action);
 	}
-
-	return action;
 }
 
 static void
@@ -263,9 +198,6 @@ toolbar_class_init (ToolbarClass *klass)
 	object_class->set_property = toolbar_set_property;
 	object_class->get_property = toolbar_get_property;
 
-	eet_class->get_action = toolbar_get_action;
-	eet_class->get_action_name = toolbar_get_action_name;
-
 	g_object_class_install_property (object_class,
                                          PROP_EPHY_WINDOW,
                                          g_param_spec_object ("EphyWindow",
@@ -352,14 +284,17 @@ toolbar_setup_actions (Toolbar *t)
 
 	action = g_object_new (EPHY_TYPE_SPINNER_ACTION,
 			       "name", "Spinner",
-			       "label", "Spinner",
+			       "label", _("Spinner"),
 			       NULL);
 	egg_action_group_add_action (t->priv->action_group, action);
 	g_object_unref (action);
 
+	/* FIXME: I'm still waiting for the exact term to 
+	 * user here from the docs team.
+	 */
 	action = g_object_new (EPHY_TYPE_LOCATION_ACTION,
 			       "name", "Location",
-			       "label", "Location",
+			       "label", _("Address Entry"),
 			       "stock_id", EPHY_STOCK_ENTRY,
 			       NULL);
 	g_signal_connect (action, "go_location",
@@ -369,7 +304,7 @@ toolbar_setup_actions (Toolbar *t)
 
 	action = g_object_new (EPHY_TYPE_FAVICON_ACTION,
 			       "name", "Favicon",
-			       "label", "Favicon",
+			       "label", _("Favicon"),
 			       "window", t->priv->window,
 			       NULL);
 	egg_action_group_add_action (t->priv->action_group, action);
@@ -377,7 +312,7 @@ toolbar_setup_actions (Toolbar *t)
 
 	action = g_object_new (EPHY_TYPE_GO_ACTION,
 			       "name", "ToolbarGo",
-			       "label", "Go",
+			       "label", _("Go"),
 			       "stock_id", GTK_STOCK_JUMP_TO,
 			       NULL);
 	g_signal_connect (action, "activate",
@@ -386,9 +321,82 @@ toolbar_setup_actions (Toolbar *t)
 	g_object_unref (action);
 }
 
+static void
+action_request_cb (EggEditableToolbar *etoolbar,
+		   char *action_name,
+		   gpointer data)
+{
+	toolbar_ensure_action (TOOLBAR (etoolbar), action_name);
+}
+
+static void
+init_bookmarks_toolbar (Toolbar *t)
+{
+	EphyToolbarsModel *model;
+	int i, n_toolbars;
+
+	model = ephy_shell_get_toolbars_model (ephy_shell);
+	n_toolbars = egg_toolbars_model_n_toolbars
+		(EGG_TOOLBARS_MODEL (model));
+
+	for (i = 0; i < n_toolbars; i++)
+	{
+		const char *t_name;
+
+		t_name = egg_toolbars_model_toolbar_nth
+			(EGG_TOOLBARS_MODEL (model), i);
+		g_return_if_fail (t_name != NULL);
+
+		if (strcmp (t_name, "BookmarksBar") == 0)
+		{
+			egg_editable_toolbar_set_drag_dest
+				(EGG_EDITABLE_TOOLBAR (t),
+				 drag_targets, n_drag_targets,
+				 t_name);
+			egg_toolbars_model_set_flags
+				(EGG_TOOLBARS_MODEL (model),
+				 EGG_TB_MODEL_NOT_REMOVABLE, i);
+		}
+	}
+}
+
+static void
+update_toolbar_remove_flag (EphyToolbarsModel *model, gpointer data)
+{
+	int i, n_toolbars;
+	int not_removable = 0;
+
+	n_toolbars = egg_toolbars_model_n_toolbars
+		(EGG_TOOLBARS_MODEL (model));
+
+	/* If there is only one toolbar and the bookmarks bar */
+	if (n_toolbars <= 2)
+	{
+		not_removable = EGG_TB_MODEL_NOT_REMOVABLE;
+	}
+
+	for (i = 0; i < n_toolbars; i++)
+	{
+		const char *t_name;
+
+		t_name = egg_toolbars_model_toolbar_nth
+			(EGG_TOOLBARS_MODEL (model), i);
+		g_return_if_fail (t_name != NULL);
+
+		if (!(strcmp (t_name, "BookmarksBar") == 0))
+		{
+			egg_toolbars_model_set_flags
+				(EGG_TOOLBARS_MODEL (model),
+				 not_removable, i);
+		}
+	}
+}
+
 static void
 toolbar_set_window (Toolbar *t, EphyWindow *window)
 {
+	EphyToolbarsModel *model;
+
 	g_return_if_fail (t->priv->window == NULL);
 
 	t->priv->window = window;
@@ -397,7 +405,51 @@ toolbar_set_window (Toolbar *t, EphyWindow *window)
 	toolbar_setup_actions (t);
 	egg_menu_merge_insert_action_group (t->priv->ui_merge,
 					    t->priv->action_group, 1);
-	g_object_set (t, "MenuMerge", t->priv->ui_merge, NULL);
+
+	g_signal_connect (t, "action_request",
+			  G_CALLBACK (action_request_cb),
+			  NULL);
+
+	model = ephy_shell_get_toolbars_model (ephy_shell);
+	g_signal_connect (EGG_TOOLBARS_MODEL (model), "toolbar_added",
+			  G_CALLBACK (update_toolbar_remove_flag),
+			  NULL);
+	g_signal_connect (EGG_TOOLBARS_MODEL (model), "toolbar_removed",
+			  G_CALLBACK (update_toolbar_remove_flag),
+			  NULL);
+	g_object_set (G_OBJECT (t),
+		      "ToolbarsModel", model,
+		      "MenuMerge", t->priv->ui_merge,
+		      NULL);
+	init_bookmarks_toolbar (t);
+}
+
+static void
+remove_action (EphyToolbarsModel *model,
+	       const char *action_name)
+{
+	int i, n_toolbars, l, n_items;
+
+	n_toolbars = egg_toolbars_model_n_toolbars
+		(EGG_TOOLBARS_MODEL (model));
+
+	for (i = 0; i < n_toolbars; i++)
+	{
+		n_items = egg_toolbars_model_n_items (EGG_TOOLBARS_MODEL (model), i);
+		for (l = 0; l < n_items; l++)
+		{
+			const char *i_name;
+			gboolean sep;
+
+			i_name = egg_toolbars_model_item_nth
+				(EGG_TOOLBARS_MODEL (model), i, l, &sep);
+			if (!sep && strcmp (action_name, i_name) == 0)
+			{
+				egg_toolbars_model_remove_item
+					(EGG_TOOLBARS_MODEL (model), i, l);
+			}
+		}
+	}
 }
 
 static void
@@ -407,12 +459,15 @@ topic_remove_cb (EphyBookmarks *eb,
 {
 	EggAction *action;
 	char *name;
+	EphyToolbarsModel *model;
+
+	model = ephy_shell_get_toolbars_model (ephy_shell);
 
 	name = g_strdup_printf ("GoTopicId%ld", id);
 	action = egg_action_group_get_action (t->priv->action_group, name);
 	if (action)
 	{
-		egg_toolbars_group_remove_action (toolbars_group, action->name);
+		remove_action (model, action->name);
 		egg_action_group_remove_action (t->priv->action_group, action);
 	}
 
@@ -426,12 +481,15 @@ bookmark_remove_cb (EphyBookmarks *eb,
 {
 	EggAction *action;
 	char *name;
+	EphyToolbarsModel *model;
+
+	model = ephy_shell_get_toolbars_model (ephy_shell);
 
 	name = g_strdup_printf ("GoBookmarkId%ld", id);
 	action = egg_action_group_get_action (t->priv->action_group, name);
 	if (action)
 	{
-		egg_toolbars_group_remove_action (toolbars_group, action->name);
+		remove_action (model, action->name);
 		egg_action_group_remove_action (t->priv->action_group, action);
 	}
 
@@ -441,36 +499,19 @@ bookmark_remove_cb (EphyBookmarks *eb,
 static void
 toolbar_init (Toolbar *t)
 {
+	EphyBookmarks *bookmarks;
+
         t->priv = g_new0 (ToolbarPrivate, 1);
 
 	t->priv->window = NULL;
 	t->priv->ui_merge = NULL;
 	t->priv->visibility = TRUE;
 
-	egg_editable_toolbar_add_drag_type (EGG_EDITABLE_TOOLBAR (t),
-					    EPHY_DND_TOPIC_TYPE);
-	egg_editable_toolbar_add_drag_type (EGG_EDITABLE_TOOLBAR (t),
-					    EPHY_DND_URL_TYPE);
-
-	if (toolbars_group == NULL)
-	{
-		char *user;
-		EphyBookmarks *bookmarks;
-
-		user = g_build_filename (ephy_dot_dir (), "toolbar.xml", NULL);
-		toolbars_group = egg_toolbars_group_new ();
-		egg_toolbars_group_set_source
-			(toolbars_group, ephy_file ("epiphany-toolbar.xml"), user);
-		g_free (user);
-
-		bookmarks = ephy_shell_get_bookmarks (ephy_shell);
-		g_signal_connect (bookmarks, "bookmark_remove",
-				  G_CALLBACK (bookmark_remove_cb), t);
-		g_signal_connect (bookmarks, "topic_remove",
-				  G_CALLBACK (topic_remove_cb), t);
-	}
-
-	g_object_set (t, "ToolbarsGroup", toolbars_group, NULL);
+	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
+	g_signal_connect (bookmarks, "bookmark_remove",
+			  G_CALLBACK (bookmark_remove_cb), t);
+	g_signal_connect (bookmarks, "topic_remove",
+			  G_CALLBACK (topic_remove_cb), t);
 }
 
 static void
@@ -529,11 +570,22 @@ toolbar_edit_location (Toolbar *t)
 		(EPHY_LOCATION_ENTRY(location));
 }
 
+static void
+location_finished_cb (GtkWidget *location, GtkWidget *toolbar)
+{
+	gtk_widget_hide (toolbar);
+
+	g_signal_handlers_disconnect_by_func (G_OBJECT (location),
+                                              G_CALLBACK (location_finished_cb),
+                                              toolbar);
+}
+
 void
 toolbar_activate_location (Toolbar *t)
 {
 	EggAction *action;
 	GtkWidget *location;
+	GtkWidget *location_tb;
 
 	action = egg_action_group_get_action
 		(t->priv->action_group, "Location");
@@ -541,6 +593,16 @@ toolbar_activate_location (Toolbar *t)
 		(EPHY_LOCATION_ACTION (action));
 	g_return_if_fail (location != NULL);
 
+	location_tb = gtk_widget_get_ancestor (location, EGG_TYPE_TOOLBAR);
+	g_return_if_fail (location_tb != NULL);
+
+	if (!GTK_WIDGET_VISIBLE (location_tb))
+	{
+		g_signal_connect (location, "finished",
+				  G_CALLBACK (location_finished_cb), location_tb);
+		gtk_widget_show (location_tb);
+	}
+
 	ephy_location_entry_activate
 		(EPHY_LOCATION_ENTRY(location));
 }
@@ -645,3 +707,50 @@ toolbar_update_navigation_actions (Toolbar *t, gboolean back, gboolean forward,
 	g_object_set (action, "sensitive", !up, NULL);
 }
 
+void
+toolbar_set_visibility (Toolbar *t,
+			gboolean normal_toolbars,
+			gboolean bmk_toolbars)
+{
+	EphyToolbarsModel *model;
+	int i, n_toolbars;
+
+	model = ephy_shell_get_toolbars_model (ephy_shell);
+	n_toolbars = egg_toolbars_model_n_toolbars
+		(EGG_TOOLBARS_MODEL (model));
+
+	for (i = 0; i < n_toolbars; i++)
+	{
+		const char *t_name;
+
+		t_name = egg_toolbars_model_toolbar_nth
+			(EGG_TOOLBARS_MODEL (model), i);
+		g_return_if_fail (t_name != NULL);
+		if (strcmp (t_name, "BookmarksBar") == 0)
+		{
+			if (bmk_toolbars)
+			{
+				egg_editable_toolbar_show
+					(EGG_EDITABLE_TOOLBAR (t), t_name);
+			}
+			else
+			{
+				egg_editable_toolbar_hide
+					(EGG_EDITABLE_TOOLBAR (t), t_name);
+			}
+		}
+		else
+		{
+			if (normal_toolbars)
+			{
+				egg_editable_toolbar_show
+					(EGG_EDITABLE_TOOLBAR (t), t_name);
+			}
+			else
+			{
+				egg_editable_toolbar_hide
+					(EGG_EDITABLE_TOOLBAR (t), t_name);
+			}
+		}
+	}
+}
