@@ -16,11 +16,6 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/*
- * 25 Mar 2003: Added support for help manual : Patanjali Somayaji
- * 						(patanjali@codito.com)
- */
-
 #include <config.h>
 
 #include "ephy-shell.h"
@@ -40,6 +35,7 @@
 #include "ephy-file-helpers.h"
 #include "toolbar.h"
 #include "ephy-state.h"
+#include "ephy-gui.h"
 
 #include <string.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
@@ -47,11 +43,14 @@
 #include <bonobo/bonobo-i18n.h>
 #include <libgnomeui/gnome-about.h>
 #include <libgnomeui/gnome-stock-icons.h>
-#include <libgnome/gnome-help.h>
 #include <libgnome/gnome-program.h>
-#include <gtk/gtkmessagedialog.h>
 #include <gtk/gtkeditable.h>
 
+enum
+{
+	RESPONSE_ADD_TOOLBAR
+};
+
 void
 window_cmd_edit_find (EggAction *action,
 		      EphyWindow *window)
@@ -277,16 +276,17 @@ window_cmd_file_bookmark_page (EggAction *action,
 	icon = ephy_tab_get_favicon_url (tab);
 
 	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
-	new_bookmark = ephy_new_bookmark_new
-		(bookmarks, GTK_WINDOW (window), location);
-	ephy_new_bookmark_set_title
-		(EPHY_NEW_BOOKMARK (new_bookmark), title);
-	ephy_new_bookmark_set_icon
-		(EPHY_NEW_BOOKMARK (new_bookmark), icon);
-	g_signal_connect (G_OBJECT (new_bookmark), "response",
-			  G_CALLBACK (ephy_new_bookmark_response_cb),
-			  NULL);
-	gtk_widget_show (new_bookmark);
+	if (ephy_new_bookmark_is_unique (bookmarks, GTK_WINDOW (window),
+					 location))
+	{
+		new_bookmark = ephy_new_bookmark_new
+			(bookmarks, GTK_WINDOW (window), location);
+		ephy_new_bookmark_set_title
+			(EPHY_NEW_BOOKMARK (new_bookmark), title);
+		ephy_new_bookmark_set_icon
+			(EPHY_NEW_BOOKMARK (new_bookmark), icon);
+		gtk_widget_show (new_bookmark);
+	}
 }
 
 void
@@ -634,36 +634,39 @@ void
 window_cmd_edit_prefs (EggAction *action,
 		       EphyWindow *window)
 {
-	GtkDialog *dialog;
+	EphyDialog *dialog;
 
-	dialog = prefs_dialog_new ();
-	prefs_dialog_show_page (PREFS_DIALOG(dialog),
-				PREFS_PAGE_GENERAL);
-	gtk_window_set_transient_for (GTK_WINDOW (dialog),
-				      GTK_WINDOW (window));
-	gtk_widget_show (GTK_WIDGET(dialog));
+	dialog = prefs_dialog_new (GTK_WIDGET (window));
+	ephy_dialog_show (dialog);
 }
 
 static void
 toolbar_editor_destroy_cb (GtkWidget *tbe,
 			   Toolbar *t)
 {
-	GtkWidget *window;
-
-	window = gtk_widget_get_toplevel (GTK_WIDGET (t));
-
 	egg_editable_toolbar_set_edit_mode (EGG_EDITABLE_TOOLBAR (t), FALSE);
-	ephy_window_update_all_controls (EPHY_WINDOW (window));
 }
 
 static void
-toolbar_editor_help_cb (GtkDialog          *dialog,
-		    	gint                response_id,
-		    	EggToolbarEditor   *editor)
+toolbar_editor_response_cb (GtkDialog  *dialog,
+			    gint response_id,
+			    gpointer data)
 {
+	EphyToolbarsModel *model;
+	int n;
+
 	switch (response_id)
 	{
-    	case GTK_RESPONSE_HELP:
+	case GTK_RESPONSE_CLOSE:
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		break;
+	case RESPONSE_ADD_TOOLBAR:
+		model = ephy_shell_get_toolbars_model (ephy_shell);
+		n = egg_toolbars_model_n_toolbars (EGG_TOOLBARS_MODEL (model));
+		egg_toolbars_model_add_toolbar (EGG_TOOLBARS_MODEL (model),
+						n - 1, "UserCreated");
+		break;
+	case GTK_RESPONSE_HELP:
 		/* FIXME: Connect toolbar editor help */
 		break;
 	}
@@ -676,29 +679,42 @@ window_cmd_edit_toolbar (EggAction *action,
 	GtkWidget *editor;
 	EphyToolbarsModel *model;
 	Toolbar *t;
+	GtkWidget *dialog;
 
 	model = ephy_shell_get_toolbars_model (ephy_shell);
 	t = ephy_window_get_toolbar (window);
 
+	dialog = gtk_dialog_new ();
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Toolbar Editor"));
+        gtk_window_set_transient_for (GTK_WINDOW (dialog),
+				      GTK_WINDOW (window));
+
 	editor = egg_toolbar_editor_new
-		(GTK_WINDOW (window),
-		 EGG_MENU_MERGE (window->ui_merge),
+		(EGG_MENU_MERGE (window->ui_merge),
 		 EGG_TOOLBARS_MODEL (model));
-	gtk_dialog_add_button (GTK_DIALOG (editor),
-			       GTK_STOCK_HELP, GTK_RESPONSE_HELP);
-	ephy_state_add_window (editor,
-			       "toolbar_editor",
-		               500, 330);
+	egg_toolbar_editor_load_actions (EGG_TOOLBAR_EDITOR (editor),
+					 ephy_file ("epiphany-toolbar.xml"));
+	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), editor);
 	g_signal_connect (editor, "destroy",
 			  G_CALLBACK (toolbar_editor_destroy_cb),
 			  t);
-	g_signal_connect (G_OBJECT (editor), "response",
-			  G_CALLBACK (toolbar_editor_help_cb),
-			  editor);
-	egg_toolbar_editor_load_actions (EGG_TOOLBAR_EDITOR (editor),
-					 ephy_file ("epiphany-toolbar.xml"));
 	gtk_widget_show (editor);
 
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       _("_Add a New Toolbar"), RESPONSE_ADD_TOOLBAR);
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       GTK_STOCK_HELP, GTK_RESPONSE_HELP);
+	g_signal_connect (G_OBJECT (dialog), "response",
+			  G_CALLBACK (toolbar_editor_response_cb), NULL);
+	ephy_state_add_window (dialog,
+			       "toolbar_editor",
+		               500, 330,
+			       EPHY_STATE_WINDOW_SAVE_SIZE);
+	gtk_widget_show (dialog);
+
 	egg_editable_toolbar_set_edit_mode (EGG_EDITABLE_TOOLBAR (t), TRUE);
 }
 
@@ -706,28 +722,7 @@ void
 window_cmd_help_contents (EggAction *action,
 			 EphyWindow *window)
 {
-	GError *err;
-
-	err = NULL;
-	gnome_help_display ("epiphany", NULL, &err);
-
-	if (err != NULL) 
-	{
-		GtkWidget *dialog;
-		dialog = gtk_message_dialog_new (GTK_WINDOW (window),
-				GTK_DIALOG_DESTROY_WITH_PARENT,
-				GTK_MESSAGE_ERROR,
-				GTK_BUTTONS_CLOSE,
-				_("Could not display help: %s"), err->message);
-
-		g_signal_connect (G_OBJECT (dialog), "response",
-			G_CALLBACK (gtk_widget_destroy),
-			NULL);
-
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-		gtk_widget_show (dialog);
-		g_error_free (err);	
-	}
+	ephy_gui_help (GTK_WINDOW (window), "epiphany", NULL);
 }
 
 void
@@ -741,11 +736,14 @@ window_cmd_help_about (EggAction *action,
 
 	static gchar *authors[] = {
 		"Marco Pesenti Gritti <mpeseng@tin.it>",
+		"Xan Lopez <xan@masilla.org>",
+		"David Bordoley <bordoley@msu.edu>",
 		NULL
 	};
 
 	gchar *documenters[] = {
 		"Patanjali Somayaji <patanjali@codito.com>",
+		"David Bordoley <bordoley@msu.edu>",
 		NULL
 	};
 
@@ -848,10 +846,8 @@ window_cmd_tabs_move_left  (EggAction *action,
 		GtkWidget *child;
 
 		child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), page);
-		gtk_notebook_reorder_child (GTK_NOTEBOOK (notebook), child, page - 1);
+		ephy_notebook_move_page (EPHY_NOTEBOOK (notebook), NULL, child, page - 1);
 	}
-
-	ephy_window_update_control (window, TabsControl);
 }
 
 void window_cmd_tabs_move_right (EggAction *action,
@@ -870,10 +866,8 @@ void window_cmd_tabs_move_right (EggAction *action,
 		GtkWidget *child;
 
 		child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), page);
-		gtk_notebook_reorder_child (GTK_NOTEBOOK (notebook), child, page + 1);
+		ephy_notebook_move_page (EPHY_NOTEBOOK (notebook), NULL, child, page + 1);
 	}
-
-	ephy_window_update_control (window, TabsControl);
 }
 
 void
@@ -884,7 +878,8 @@ window_cmd_tabs_detach  (EggAction *action,
 	GtkWidget *src_page;
 	EphyWindow *new_win;
 
-	if (g_list_length (ephy_window_get_tabs (window)) <= 1) {
+	if (g_list_length (ephy_window_get_tabs (window)) <= 1)
+	{
 		return;
 	}
 
